from ehelply_bootstrapper.utils.environment import Environment


def _get_environment_name() -> str:
    """
    Maps the dev environment to test as we don't store secrets specific to dev.

    Returns:

    """
    environment_name: str = Environment.stage()
    if environment_name == "dev":
        environment_name = "test"
    return environment_name


def secret_name_database():
    """
    AWS ASM secret name for database credentials

    Returns:
        None

    """
    return "ehelply-{environment}.db-rds.ehelply-{environment}-main.ehelply-secure-microservice".format(
        environment=_get_environment_name())


def secret_name_facts_vault():
    """
    AWS ASM secret name for access to the facts vault

    Returns:
        None

    """
    return "ehelply-{environment}.microservices.ehelply-facts.vault".format(environment=_get_environment_name())


def secret_name_security_vault():
    """
    AWS ASM secret name for access to the security vault

    Returns:
        None

    """
    return "ehelply-{environment}.microservices.ehelply-security.vault".format(environment=_get_environment_name())


def secret_name_updater_vault():
    """
    AWS ASM secret name for access to the updater vault

    Returns:
        None

    """
    return "ehelply-all.microservices.ehelply-updater.vault".format(environment=_get_environment_name())


def secret_name_auth_dynamo():
    """
    AWS ASM secret name for access to auth dynamo encryption

    Returns:
        None

    """
    return "ehelply-{environment}.microservices.auth-dynamos.vault".format(environment=_get_environment_name())



from pydantic import BaseModel
from typing import List, Dict, Callable
from ehelply_bootstrapper.utils.db_encryption import DBEncryption


class eHelplySchema(BaseModel):
    """
    Extended Pydantic base model which adds helper functions for ease of use
    """

    def __init__(self, db_model=None, **kwargs):
        """
        Normal initialization of a Pydantic model.

        However, if a db_model is passed in, it will set up the Pydantic model with default values.

        Still, passing in values manually will OVERRIDE the values from the DB model.

        Args:
            db_model: SQL Alchemy model
            **kwargs:
        """
        if db_model:
            fields_from_model: dict = self.sql_to_pydantic(db_model)
            fields_from_model.update(kwargs)
            kwargs = fields_from_model

        super().__init__(**kwargs)

    def map(self, obj, whitelist: list = None):
        """
        Maps pydantic model values to another object. For example, a SQL Alchemy model.

        If whitelist is specified, it will only map those keys, otherwise, it maps all the keys

        Args:
            obj:
            whitelist:

        Returns:

        """
        my_data: dict = self.dict()
        if whitelist:
            for key in whitelist:
                if key in my_data and my_data[key] is not None:
                    setattr(obj, key, my_data[key])
        else:
            for key, value in my_data.items():
                if value is not None:
                    setattr(obj, key, value)
        return obj

    def pydantic_to_sql(self, obj, whitelist: list = None):
        """
        Alias function to provide a consistent API

        Args:
            obj:
            whitelist:

        Returns:

        """
        return self.map(obj, whitelist)

    def sql_encrypted_fields(self) -> Dict[str, str]:
        """
        Override to specify DB model fields to automatically decrypt.

        Format:
        {
            "field_name": "encryption_field_type"
        }

        Format example:
        {
            "address": "str"
        }

        Returns:

        """
        return {}

    def sql_pre_conversion(self, model: dict) -> dict:
        """
        Override this function to provide defaults and any other pre-processing that is required.

        Args:
            model:

        Returns:

        """
        return model

    def sql_to_pydantic(self, model):
        """
        Converts SQL DB Model from SQL Alchemy to this Pydantic model

        Args:
            model: SQL Alchemy model

        Returns:

        """
        model_dict: dict = model.asdict()

        for field, encrypted_type in self.sql_encrypted_fields().items():
            if field in model_dict:
                model_dict[field] = DBEncryption.decrypt(data=model_dict[field], enc_type=encrypted_type)

        return self.sql_pre_conversion(model_dict)



from ehelply_bootstrapper.utils.state import State
from ehelply_bootstrapper.utils.cryptography import Encryption
from ehelply_bootstrapper.integrations.integration import Integration
from ehelply_bootstrapper.utils.environment import Environment
from ehelply_python_experimental_sdk.configuration import Configuration as ExperimentalConfiguration
from ehelply_python_experimental_sdk.api_client import ApiClient as ExperimentalApiClient
from ehelply_python_sdk.configuration import Configuration
from ehelply_python_sdk import ApiClient

from ehelply_microservice_library.integrations.fact import get_fact_endpoint


def make_api_client(config: Configuration, include_default_headers: bool = True) -> ApiClient:
    api_client: ApiClient = ApiClient(configuration=config)
    if include_default_headers:
        api_client.default_headers["X-Access-Token"] = SDK.headers["x_access_token"]
        api_client.default_headers["X-Secret-Token"] = SDK.headers["x_secret_token"]
        api_client.default_headers["Ehelply-Project"] = SDK.headers["ehelply_project"]
    return api_client


def make_experimental_api_client(
        config: ExperimentalConfiguration,
        include_default_headers: bool = True
) -> ExperimentalApiClient:
    api_client: ExperimentalApiClient = ExperimentalApiClient(configuration=config)
    if include_default_headers:
        api_client.default_headers["X-Access-Token"] = SDK.headers["x_access_token"]
        api_client.default_headers["X-Secret-Token"] = SDK.headers["x_secret_token"]
        api_client.default_headers["Ehelply-Project"] = SDK.headers["ehelply_project"]
    return api_client


class SDK(Integration):
    headers: dict
    base_url: str
    configuration: Configuration
    experimental_configuration: ExperimentalConfiguration

    def __init__(self, service_gatekeeper_key: str) -> None:
        super().__init__("sdk")

        self.enc: Encryption = Encryption([service_gatekeeper_key.encode(Encryption.STRING_ENCODING)])

    def init(self):
        try:
            secret_token: str = State.config.m2m.auth.secret_key
            access_token: str = State.config.m2m.auth.access_key

            if len(secret_token) == 0 or len(access_token) == 0:
                State.logger.warning("SDK (M2M) credentials are not set. Check the m2m.yaml config")

            secret_token = self.enc.decrypt_str(secret_token.encode(Encryption.STRING_ENCODING))
            access_token = self.enc.decrypt_str(access_token.encode(Encryption.STRING_ENCODING))

            # Setup SDK
            SDK.make_sdk(access_token, secret_token)

        except:
            SDK.make_sdk("", "")
            State.logger.severe(
                "SDK (M2M) credentials are invalid. Ensure they are encrypted. Check the m2m.yaml config")

    @staticmethod
    def make_sdk(access_token: str, secret_token: str):
        host: str = "https://api.prod.ehelply.com"

        if Environment.is_dev() or Environment.is_test():
            host = "https://api.test.ehelply.com"

        SDK.base_url = host

        SDK.configuration = Configuration(
            host=host
        )

        SDK.experimental_configuration = ExperimentalConfiguration(
            host=host
        )

        SDK.headers = {
            "x_access_token": access_token,
            "x_secret_token": secret_token,
            "ehelply_project": "ehelply-cloud"
        }

    @staticmethod
    def make_notes_client(include_default_headers: bool = True) -> ApiClient:
        config = Configuration(
            host=get_fact_endpoint("ehelply-notes")
        )
        return make_api_client(config, include_default_headers)

    @staticmethod
    def make_meta_client(include_default_headers: bool = True) -> ApiClient:
        config = Configuration(
            host=get_fact_endpoint("ehelply-meta")
        )
        return make_api_client(config, include_default_headers)

    @staticmethod
    def make_experimental_notes_client(include_default_headers: bool = True) -> ExperimentalApiClient:
        config = ExperimentalConfiguration(
            host=get_fact_endpoint("ehelply-notes")
        )
        return make_experimental_api_client(config, include_default_headers)

    @staticmethod
    def make_experimental_meta_client(include_default_headers: bool = True) -> ExperimentalApiClient:
        config = ExperimentalConfiguration(
            host=get_fact_endpoint("ehelply-meta")
        )
        return make_experimental_api_client(config, include_default_headers)
